"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from __future__ import absolute_import

import os
import tensorflow as tf
from absl import app
from absl import flags

from tcav import tcav
from tcav import activation_generator
from tcav import utils
import resnet_model_wrapper

flags.DEFINE_string('model', 'obj',
                    'Name of the model. Should exist in models directory')
FLAGS = flags.FLAGS


def compute_tcav_scores(target='dog_bedroom',
                        random_counterpart='random_counter_part',
                        concepts=['dog_scene']):
  """Compute TCAV scores of a given list of concepts for a ResNet model.

  Computation is done for each block layer and the logit layer.
  """

  base_dir = os.getcwd()
  model_dir = os.path.join(base_dir, 'models', FLAGS.model)
  img_dir = os.path.join(base_dir, 'data/tcav')
  if FLAGS.model == 'obj':
    cat_fpath = os.path.join(base_dir, 'data/tcav', 'obj_categories.txt')
  else:
    cat_fpath = os.path.join(base_dir, 'data/tcav', 'scene_categories.txt')
  working_dir = os.path.join(base_dir, 'tcav_working_dir', FLAGS.model)
  if not tf.gfile.Exists(working_dir):
    tf.gfile.MakeDirs(working_dir)
    tf.gfile.MakeDirs(os.path.join(working_dir, 'activations'))
    tf.gfile.MakeDirs(os.path.join(working_dir, 'cavs'))

  sess = utils.create_session()
  tcav_model_wrapper = resnet_model_wrapper.ResNetModelWrapper(
      sess, model_dir, cat_fpath)
  act_gen = activation_generator.ImageActivationGenerator(
      tcav_model_wrapper,
      img_dir,
      os.path.join(working_dir, 'activations'),
      max_examples=100,
      normalize_image=False)

  bottlenecks = [
      'block_layer1', 'block_layer2', 'block_layer3', 'block_layer4', 'logit'
  ]
  for bottleneck in bottlenecks:
    mytcav = tcav.TCAV(
        sess,
        target,
        concepts, [bottleneck],
        act_gen, [0.1],
        random_counterpart,
        cav_dir=os.path.join(working_dir, 'cavs'),
        num_random_exp=2)
    results = mytcav.run()
    utils.print_results(results, random_counterpart='random_counter_part')


def main(argv):  # pylint: disable=unused-argument
  compute_tcav_scores()


if __name__ == '__main__':
  app.run(main)



"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from tensorflow.python.platform import googletest
from metrics import load_pos_neg_attr
from metrics import compute_and_save_attr
import random
import os

class PipelineTest(googletest.TestCase):

  def test_compute_and_load_attr_correct(self):
      models = ['obj', 'scene']
      datas = ['scene', 'obj']
      num_imgs = 3

      # Sample num_imgs images to compute attributions
      random.seed(0)
      indices = random.sample(range(100), num_imgs)
      
      compute_and_save_attr(models[0], datas[0], indices, 10)
      compute_and_save_attr(models[1], datas[1], indices, 10)

      sal_dir_pos = os.path.join(os.getcwd(), 'sal_scratch', models[0] + '-' + datas[0])
      sal_dir_neg = os.path.join(os.getcwd(), 'sal_scratch', models[1] + '-' + datas[1])

      # Saliency maps directory contains the right files
      self.assertEqual(len(os.listdir(sal_dir_pos)), num_imgs)
      self.assertEqual(len(os.listdir(sal_dir_neg)), num_imgs)
      self.assertEqual(sorted(os.listdir(sal_dir_pos)), sorted(os.listdir(sal_dir_neg)))

      attr_pos, attr_neg = load_pos_neg_attr(models[0], datas[0],
                                             models[1], datas[1])

      attr_dir_pos = os.path.join(os.getcwd(), 'attr_scratch', models[0] + '-' + datas[0])
      attr_dir_neg = os.path.join(os.getcwd(), 'attr_scratch', models[1] + '-' + datas[1])

      # Attributions directory contains the right files      
      self.assertEqual(len(os.listdir(attr_dir_pos)), num_imgs)
      self.assertEqual(len(os.listdir(attr_dir_neg)), num_imgs)
      self.assertEqual(sorted(os.listdir(attr_dir_pos)), sorted(os.listdir(attr_dir_neg)))

      # Attribution is higher in obj model than in scene model for these examples
      self.assertEqual(attr_pos.shape, attr_neg.shape)
      method_indices = [6, 0, 1, 4, 5, 10, 2, 8]
      for i in range(num_imgs):
          for j in method_indices:
              self.assertLess(attr_neg[i,j], attr_pos[i, j])    

if __name__ == '__main__':
    googletest.main()



"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import tensorflow as tf
import numpy as np
import PIL.Image
import os
from multiprocessing import dummy as multiprocessing
import saliency

ROW_MIN = 0
COL_MIN = 1
ROW_MAX = 2
COL_MAX = 3

# Same as in https://github.com/tensorflow/models/blob/master/official/resnet/imagenet_preprocessing.py
_R_MEAN = 123.68
_G_MEAN = 116.78
_B_MEAN = 103.94
_CHANNEL_MEANS = [_R_MEAN, _G_MEAN, _B_MEAN]

SAL_DIR = 'sal_scratch'
ATTR_DIR = 'attr_scratch'
RESNET_SHAPE = (224, 224)
IMG_SHAPE = (128, 128)


def load_imgs(fnames, num_threads=1, shape=RESNET_SHAPE):
  """Load and preprocess images for ResNet by subtracting means."""

  def load_img(f):
    img = PIL.Image.open(tf.gfile.Open(f, 'rb'))
    img = img.convert('RGB').resize(shape, PIL.Image.BILINEAR)
    channel_means = np.expand_dims(np.expand_dims(_CHANNEL_MEANS, 0), 0)
    img_arr = np.array(img, dtype=np.float32) - channel_means.astype(np.float32)
    return img_arr

  pool = multiprocessing.Pool(num_threads)
  return pool.map(load_img, fnames)


def visualize_pos_attr(image_3d, percentile=99):
  """Returns a 3D tensor as a grayscale 2D tensor by summing the positive attributions of a 3D tensor across axis=2, and then clips values at a given percentile."""

  image_2d = np.sum(image_3d.clip(min=0), axis=2)
  vmax = np.percentile(image_2d, percentile)
  vmin = np.min(image_2d)
  return np.clip((image_2d - vmin) / (vmax - vmin), 0, 1)


def single_attr(map_2d, loc, obj_mask):
  """Given a 2D saliency map, the location of an object, and the binary mask of that object, compute the attribution of the object by averaging over its pixel-wise attributions."""

  obj_mask_resized = np.array(
      PIL.Image.fromarray(obj_mask).resize(
          (loc[COL_MAX] - loc[COL_MIN], loc[ROW_MAX] - loc[ROW_MIN]),
          PIL.Image.BILINEAR))
  avg = np.sum(
      map_2d[:, loc[ROW_MIN]:loc[ROW_MAX], loc[COL_MIN]:loc[COL_MAX]] *
      obj_mask_resized,
      axis=(-1, -2)) / np.count_nonzero(obj_mask_resized)
  return avg


def compute_and_save_attr(model, data, indices, num_threads):
  """Given the name of a model and a set of data, select a set of images based on provided indices, and compute and save their saliency maps and object attributions."""

  base_dir = os.getcwd()
  data_dir = os.path.join(base_dir, 'data', data, 'val')
  model_dir = os.path.join(base_dir, 'models', model)
  sal_output_dir = os.path.join(base_dir, SAL_DIR, model + '-' + data)
  attr_output_dir = os.path.join(base_dir, ATTR_DIR, model + '-' + data)
  if not tf.gfile.Exists(sal_output_dir):
    tf.gfile.MakeDirs(sal_output_dir)
  if not tf.gfile.Exists(attr_output_dir):
    tf.gfile.MakeDirs(attr_output_dir)

  img_names = [sorted(tf.gfile.ListDirectory(data_dir))[i] for i in indices]
  img_paths = [os.path.join(data_dir, img_name) for img_name in img_names]
  imgs = load_imgs(img_paths, num_threads)

  input_name = 'input_tensor:0'
  logit_name = 'resnet_model/final_dense:0'
  conv_name = 'resnet_model/block_layer4:0'

  with tf.Session(graph=tf.Graph()) as sess:
    tf.saved_model.loader.load(sess, ['serve'], model_dir)
    graph = tf.get_default_graph()
    input_tensor = graph.get_tensor_by_name(input_name)
    logit_tensor = graph.get_tensor_by_name(logit_name)
    neuron_selector = tf.placeholder(tf.int32)
    y = logit_tensor[:, neuron_selector]
    pred_tensor = tf.argmax(logit_tensor, 1)

    vg = saliency.GradientSaliency(graph, sess, y, input_tensor)
    gb = saliency.GuidedBackprop(graph, sess, y, input_tensor)
    ig = saliency.IntegratedGradients(graph, sess, y, input_tensor)
    gc = saliency.GradCam(graph, sess, y, input_tensor,
                          graph.get_tensor_by_name(conv_name))

    def single_map(img, img_name):
      pred = sess.run(pred_tensor, feed_dict={input_tensor: [img]})[0]
      vg_mask = vg.GetMask(img, feed_dict={neuron_selector: pred})
      # *s is SmoothGrad
      vgs_mask = vg.GetSmoothedMask(
          img, feed_dict={neuron_selector: pred})
      gb_mask = gb.GetMask(img, feed_dict={neuron_selector: pred})
      gbs_mask = gb.GetSmoothedMask(
          img, feed_dict={neuron_selector: pred})
      baseline = np.zeros(img.shape) - np.expand_dims(
          np.expand_dims(_CHANNEL_MEANS, 0), 0)
      ig_mask = ig.GetMask(
          img, feed_dict={neuron_selector: pred}, x_baseline=baseline)
      igs_mask = ig.GetSmoothedMask(
          img,
          feed_dict={neuron_selector: pred},
          x_baseline=baseline)
      gc_mask = gc.GetMask(img, feed_dict={neuron_selector: pred})
      gcs_mask = gc.GetSmoothedMask(
          img, feed_dict={neuron_selector: pred})
      # gbgc is guided GradCam
      gbgc_mask = gb_mask * gc_mask
      gbgcs_mask = gbs_mask * gcs_mask
      # Also include gradient x input
      masks = np.array([
          vg_mask, vgs_mask, gb_mask, gbs_mask, ig_mask, igs_mask, gc_mask,
          gcs_mask, gbgc_mask, gbgcs_mask, vg_mask * img, vgs_mask * img
      ])
      return masks, pred

    sal_maps = []
    preds = []
    for img, img_name in zip(imgs, img_names):
      sal_path = tf.gfile.Glob(
          os.path.join(sal_output_dir, img_name[:-4] + '*'))
      if len(sal_path) > 0:
        sal_maps.append(np.load(tf.gfile.GFile(sal_path[0], 'rb')))
        preds.append(sal_path[0].split('_')[-1])
        tf.logging.info('Loaded saliency maps for {}.'.format(img_name))
      else:
        masks, pred = single_map(img, img_name)
        sal_maps.append(masks)
        preds.append(pred)
        out_path = os.path.join(sal_output_dir, img_name[:-4] + '_' + str(pred))
        np.save(tf.gfile.GFile(out_path, 'w'), masks)
        tf.logging.info('Saved saliency maps for {}.'.format(img_name))

  # Locate the objects, convert 3D saliency maps to 2D, and compute
  # the attributions of the object segments by averaging over the
  # per-pixel attributions of the objects.
  loc_fpath = os.path.join(base_dir, 'data', data, 'val_loc.txt')
  lines = [tf.gfile.Open(loc_fpath).readlines()[i] for i in indices]
  locs = np.array([[
      int(int(l) * float(RESNET_SHAPE[0]) / IMG_SHAPE[0])
      for l in line.rstrip('\n').split(' ')[-1].split(',')
  ]
                   for line in lines])
  pool = multiprocessing.Pool(num_threads)
  maps_3d = np.array(sal_maps).reshape(-1, RESNET_SHAPE[0], RESNET_SHAPE[1], 3)
  maps_2d = np.array(pool.map(visualize_pos_attr, maps_3d))
  maps_2d = maps_2d.reshape(
      len(indices), int(maps_2d.shape[0] // len(indices)), RESNET_SHAPE[0],
      RESNET_SHAPE[1])
  mask_fpath = os.path.join(base_dir, 'data', data, 'val_mask')

  if data in ['obj', 'scene', 'scene_only']:
    # MCS and IDR are evaluated on 10000 images and masks are 10x100.
    # Find the right mask.
    obj_dict = {'backpack': 0,
                'bird': 1,
                'dog': 2,
                'elephant': 3,
                'kite': 4,
                'pizza': 5,
                'stop_sign': 6,
                'toilet': 7,
                'truck': 8,
                'zebra': 9,
    }
    
    # Loading val_mask from the data directory
    masks_mat = np.load(tf.gfile.GFile(mask_fpath, 'rb'), allow_pickle=True)
    # Getting obj indices
    obj_inds = [obj_dict[i.split('.')[0].split('-')[0]] for i in img_names]
    # getting indices for a particular object class
    temp_inds = [int(i.split('.')[0][-2:]) for i in img_names]

    obj_masks = [masks_mat[obj_inds[i]*100 + temp_inds[i]]
                 for i, _ in enumerate(img_names)]

  else:
    obj_masks = [
      np.load(tf.gfile.GFile(mask_fpath, 'rb'), allow_pickle=True)[i]
      for i in indices
    ]

  attrs = []
  for i in range(len(indices)):
    attr = single_attr(maps_2d[i], locs[i], obj_masks[i])
    attrs.append(attr)
    out_path = os.path.join(attr_output_dir,
                            img_names[i][:-4] + '_' + str(preds[i]))
    np.save(tf.gfile.GFile(out_path, 'w'), attr)



"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from tcav import model
import tensorflow as tf


class ResNetModelWrapper(model.ImageModelWrapper):

  def __init__(self, sess, model_dir, categories_file):
    """
        Args:
          categories_file: file path that contains alphabetically sorted
            categories and their corresponding index. e.g. /a/abbey 0
    """
    self.model_name = "ResNet"
    self.sess = sess
    image_shape = (224, 224, 3)
    super(ResNetModelWrapper, self).__init__(image_shape)

    tf.saved_model.loader.load(sess, ["serve"], model_dir)
    self.labels = [
        line.split(" ")[0]
        for line in tf.gfile.Open(categories_file).read().splitlines()
    ]

    node_dict = {
        "input": "input_tensor",
        "block_layer1": "resnet_model/block_layer1",
        "block_layer2": "resnet_model/block_layer2",
        "block_layer3": "resnet_model/block_layer3",
        "block_layer4": "resnet_model/block_layer4",
        "logit": "resnet_model/final_dense"
    }
    self.bottlenecks_tensors = {}
    for endpoint in node_dict:
      self.bottlenecks_tensors[endpoint] = sess.graph.get_operation_by_name(
          node_dict[endpoint]).outputs[0]
    self.bottleneck_names = self.bottlenecks_tensors.keys()
    self.ends = {}
    self.ends["input"] = self.bottlenecks_tensors["input"]
    self.ends["logit"] = self.bottlenecks_tensors["logit"]
    self.ends["prediction"] = self.ends["logit"]

    with sess.graph.as_default():
      self.y_input = tf.placeholder(tf.int64, shape=[None])
      self.pred = tf.expand_dims(self.ends["prediction"][0], 0)
      self.loss = tf.reduce_mean(
          tf.nn.softmax_cross_entropy_with_logits(
              labels=tf.one_hot(self.y_input, len(self.labels)),
              logits=self.pred))
    self._make_gradient_tensors()

  def label_to_id(self, label):
    return self.labels.index(label)

  def id_to_label(self, idx):
    return self.labels[idx]



"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from os import path
from setuptools import setup, find_packages

package_dir = path.abspath(path.dirname(__file__))
with open(path.join(package_dir, 'README.md')) as f:
  long_description = f.read()

setup(
    name='bam-intp',
    version='0.0',
    description='Benchmarking attribution methods.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/google-research-datasets/bam',
    author='Google Inc.',
    author_email='opensource@google.com',
    license='Apache 2.0',
    packages=find_packages(),
    zip_safe=False,
    install_requires=[
        'numpy',
        'tensorflow',
        'Pillow>=6.0.0',
        'tcav>=0.2.1',
        'saliency>=0.0.2',
        'Cython>=0.29.12',
        'pycocotools>=2.0.0',
        'scikit-learn>=0.20.3',
    ])



"""Copyright 2019 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import tensorflow as tf
import numpy as np
import random
from multiprocessing import dummy as multiprocessing
from absl import app
from absl import flags
from attributions import compute_and_save_attr

flags.DEFINE_integer('num_imgs', 10,
                     'Number of images to compute metrics over.')
flags.DEFINE_integer(
    'num_threads', 50,
    'Number of threads to use when perform images operations.')
flags.DEFINE_integer(
    'seed', 0, 'The seed to use when randomly sample from a set of inputs.'
    'Make sure sal_scratch is empty before changing the seed.')
flags.DEFINE_boolean(
    'scratch', True,
    'Compute metrics from scratch for num_imgs example inputs.')
flags.DEFINE_list('metrics', ['MCS', 'IDR', 'IIR'],
                  'List of metrics to evaluate.')
FLAGS = flags.FLAGS

ATTR_DIR = ['attr_reprod', 'attr_scratch']
METHOD_NAMES = [
    'Vanilla Gradient', 'Gradient SmoothGrad', 'Guided Backprop',
    'Guided Backprop SmoothGrad', 'Integrated Gradient',
    'Integrated Gradient SmoothGrad', 'GradCam', 'GradCam SmoothGrad',
    'Guided GradCam', 'Guided GradCam SmoothGrad', 'Gradient x Input',
    'Gradient SmoothGrad x Input'
]

# Index into METHOD_NAMES for the desired method names. Ordered roughly
# in the best to worst performance under the BIM metric.
METHOD_INDICES = [6, 0, 1, 4, 5, 10, 2, 8]

# For MCS, IDR, and IIR, the configs correspond to the names of the model-data
# pair (e.g., model1, data1, model2, data2). For RMCS, the config corresponds
# to the prefix of the model name and the data name.
METRIC_CONFIG = {
    'MCS': ['obj', 'obj', 'scene', 'scene'],
    'IDR': ['scene', 'scene_only', 'scene', 'scene'],
    'IIR': ['scene_only', 'bamboo_forest', 'scene_only', 'bamboo_forest_patch'],
    'RMCS': ['scene', 'dog_bedroom']
}

# BIM models have 10 classes so there are 10 models for RMC testing.
NUM_RELATIVE_MODELS = 10

# BIM's dataset for IIR contains 100 images so sample up to 100.
MAX_SAMPLE_INDEX = 100

# MCS and IDR are evaluated on 10000 images, IIR and RMCS are on 100.
NUM_TOTAL_IMAGES = {
    'MCS': 10000,
    'IDR': 10000,
    'IIR': 100,
    'RMCS': 100,
}

BASE_DIR = os.getcwd()


def get_global_indices(metric):
  """When computing from scratch, generate random global image indices from a fixed seed.

  When reproducing results given attributions, enumerate all indices of
  NUM_TOTAL_IMAGES['metric'].
  """

  if not FLAGS.scratch:
    return range(NUM_TOTAL_IMAGES[metric])
  random.seed(FLAGS.seed)
  return sorted(random.sample(range(NUM_TOTAL_IMAGES[metric]), FLAGS.num_imgs))


def corr_indices(model, data, metric):
  """Given the name of a model and a set of data, return the indices of the images that are correctly classified by the model."""

  label_fpath = os.path.join(BASE_DIR, 'data', data, 'val.txt')
  labels = [
      int(l.split(' ')[-1]) for l in tf.gfile.Open(label_fpath).readlines()
  ]
  img_fnames = tf.gfile.ListDirectory(
      os.path.join(BASE_DIR, ATTR_DIR[FLAGS.scratch], model + '-' + data))
  preds = [int(p.split('_')[-1]) for p in sorted(img_fnames)]
  attr_indices = range(len(preds))
  if FLAGS.scratch:
    attr_indices = range(FLAGS.num_imgs)
  global_indices = get_global_indices(metric)
  corr = [i for i in attr_indices if preds[i] == labels[global_indices[i]]]
  return corr


def load_pos_neg_attr(model_pos, data_pos, model_neg, data_neg):
  """Load two sets of attributions from model_pos-data_pos and model_neg-data_neg.

  Filter and only return attributions of correctly classified inputs.

  Args:
      model_pos: the model name for which objects have positive attributions.
      data_pos: the data name for which objects have positive attributions.
      model_neg: the model name for which objects have negative attributions.
      data_neg: the data name for which objects have negative attributions.

  Returns:
      arrays of attributions corresponding to model_pos-data_pos and
      model_neg-data_neg pairs.
  """

  dir_pos = os.path.join(BASE_DIR, ATTR_DIR[FLAGS.scratch],
                         model_pos + '-' + data_pos)
  pool = multiprocessing.Pool(FLAGS.num_threads)
  attr_pos = np.array(
      pool.map(
          lambda f: np.load(tf.gfile.GFile(os.path.join(dir_pos, f), 'rb')),
          sorted(tf.gfile.ListDirectory(dir_pos))))
  dir_neg = os.path.join(BASE_DIR, ATTR_DIR[FLAGS.scratch],
                         model_neg + '-' + data_neg)
  attr_neg = np.array(
      pool.map(
          lambda f: np.load(tf.gfile.GFile(os.path.join(dir_neg, f), 'rb')),
          sorted(tf.gfile.ListDirectory(dir_neg))))
  if FLAGS.scratch:
    attr_pos = attr_pos[:FLAGS.num_imgs]
    attr_neg = attr_neg[:FLAGS.num_imgs]
  return attr_pos, attr_neg


def MCS(model_pos, data_pos, model_neg, data_neg, relative=False):
  """Compute the model contrast score defined as the average attribution

  difference between model_pos-data_pos and model_neg-data_neg.

  Args:
      model_pos: the model name for which objects have positive attributions.
      data_pos: the data name for which objects have positive attributions.
      model_neg: the model name for which objects have negative attributions.
      data_neg: the data name for which objects have negative attributions.
  """

  metric = 'RMCS' if relative else 'MCS'
  attr_pos, attr_neg = load_pos_neg_attr(model_pos, data_pos, model_neg,
                                         data_neg)
  corr_pos = corr_indices(model_pos, data_pos, metric)
  corr_neg = corr_indices(model_neg, data_neg, metric)
  corr = [i for i in corr_pos if i in corr_neg]
  for j in METHOD_INDICES:
    print(','.join(
        map(str, [METHOD_NAMES[j],
                  np.mean((attr_pos - attr_neg)[corr, j])])))


def IDR(model, data_pos, data_neg):
  """Compute the input dependence rate defined as the percentage of examples where data_pos is attributed higher than data_neg.

  Args:
      model: name of the model being evaluated.
      data_pos: the data name for positive attributions.
      data_neg: the data name for negative attributions.
  """

  attr_pos, attr_neg = load_pos_neg_attr(model, data_pos, model, data_neg)
  corr_pos = corr_indices(model, data_pos, 'IDR')
  corr_neg = corr_indices(model, data_neg, 'IDR')
  corr = [i for i in corr_pos if i in corr_neg]
  for j in METHOD_INDICES:
    count = sum(d > 0 for d in (attr_pos - attr_neg)[corr, j])
    print(','.join(map(str, [METHOD_NAMES[j], count / float(len(corr))])))


def IIR(model, data, data_patch, threashold=0.1):
  """Compute the input independence rate defined as the percentage of examples where the difference between data and data_patch is less than threshold.

  Args:
      model: name of the model being evaluated.
      data: name of the data directory that contains scene-only images.
      data_patch: name of the data directory that contains functionally
        insignificant object patches.
  """

  attr, attr_patch = load_pos_neg_attr(model, data, model, data_patch)
  corr = corr_indices(model, data, 'IIR')
  corr_patch = corr_indices(model, data_patch, 'IIR')
  corr = [i for i in corr if i in corr_patch]
  for j in METHOD_INDICES:
    diff = abs(attr[corr, j] - attr_patch[corr, j])
    count = sum(diff < threashold * attr[corr, j])
    print(','.join(map(str, [METHOD_NAMES[j], count / float(len(corr))])))


def main(argv):
  for metric in FLAGS.metrics:
    print('Results for {}:'.format(metric))

    global_indices = get_global_indices(metric)
    if metric == 'RMCS':
      model_prefix, data = METRIC_CONFIG[metric]
      if FLAGS.scratch:
        for i in range(1, NUM_RELATIVE_MODELS + 1):
          compute_and_save_attr(model_prefix + str(i), data, global_indices,
                                FLAGS.num_threads)
      for i in range(1, NUM_RELATIVE_MODELS):
        print('MCS between', model_prefix + str(i), 'and',
              model_prefix + str(NUM_RELATIVE_MODELS))
        MCS(model_prefix + str(i), data,
            model_prefix + str(NUM_RELATIVE_MODELS), data, relative=True)
    else:
      model1, data1, model2, data2 = METRIC_CONFIG[metric]
      if FLAGS.scratch:
        compute_and_save_attr(model1, data1, global_indices, FLAGS.num_threads)
        compute_and_save_attr(model2, data2, global_indices, FLAGS.num_threads)
      if metric == 'MCS':
        MCS(model1, data1, model2, data2)
      if metric == 'IDR':
        IDR(model1, data1, data2)
      if metric == 'IIR':
        IIR(model1, data1, data2)


if __name__ == '__main__':
  app.run(main)
